local addon, private = ...

-- Define our tooltip frames
local cmiContext = UI.CreateContext("cmiTooltip")
local cmiTooltipFrame = UI.CreateFrame("Texture", "cmiTooltipFrame", cmiContext)
local cmiTooltipText = UI.CreateFrame("Text", "cmiTooltipText", cmiContext)

-- Initialize our tooltip frames so they're displayed and layered properly.  Hide it since we don't need it initially
cmiContext:SetStrata("topmost")
cmiTooltipFrame:SetTexture("Rift", "ItemToolTip_I75.dds")
cmiTooltipFrame:SetLayer(1)
cmiTooltipText:SetLayer(2)
cmiContext:SetVisible(false)

-- Padding variables so that we can change the position of the text in the frame
local CMIXPADDING = 10
local CMIYPADDING = 5

-- TODO: Delete this, this is just for me and I stole it from the interwebs.
function print_r (t, indent, done)
  done = done or {}
  indent = indent or ''
  local nextIndent -- Storage for next indentation value
  for key, value in pairs (t) do
    if type (value) == "table" and not done [value] then
      nextIndent = nextIndent or
          (indent .. string.rep(' ',string.len(tostring (key))+2))
          -- Shortcut conditional allocation
      done [value] = true
      print (indent .. "[" .. tostring (key) .. "] => Table {");
      print  (nextIndent .. "{");
      print_r (value, nextIndent .. string.rep(' ',2), done)
      print  (nextIndent .. "}");
    else
      print  (indent .. "[" .. tostring (key) .. "] => " .. tostring (value).."")
    end
  end
end

-----------------------------------------------------------
-- Tooltip analyzer to figure out what we're hovering over
-----------------------------------------------------------
function cmiAnalyzeItem()
    -- Set up some junk variables for the tooltip string
    local itemString = ""
    local itemCount = 0

    -- Clear our tooltip, regardless of what type of item it is.  Can re-show it later if we need to
    cmiContext:SetVisible(false)

    -- Inspect the tooltip information, figure out what we're hovering over
    itemType, itemId = Inspect.Tooltip()
	--testing here..
   --print (itemType, itemId)

    -- If this isn't of type "item" or "itemtype" then we don't need to do anything, just exit the function
    if (itemType ~= "item" and itemType ~= "itemtype") then return end
    
    itemInfo = Inspect.Item.Detail(itemId)

    -- Still not sure why I have to do this, but resourced and clickables on the ground are registered as "item"
    --  but do not return anything from Inspect.Item.Detail
    if (itemInfo == nil) then return end
--testing here..
    --print(itemInfo.category)

    -- There are some items that have no "category" assigned to them in the game.  This is to avoid errors.  I'm
    --  sure I will have to un-cheese this later when I implement "crafting ingredient" into the lookup table.
    if (itemInfo.category == nil) then return end

    -- Only do stuff if it's a crafting material, and a crafting ingredient.  No need to search a DB for non-crafting stuff (yay optomizing?)
    if (string.find(itemInfo.category, "crafting material") == nil) and (string.find(itemInfo.category, "crafting ingredient") == nil) then return end
--
    -- itemInfo.category holds a string similar to "crafting material fish".  We're getting everything after "crafting "
    craftingType = string.sub(itemInfo.category, 10)
--testing here..
    --print (craftingType)

    -- If this returns nil, that means for some reason I don't have a crafting type in the database and hopefully people let me know!
    if (cmiData[craftingType] == nil) then print (cmiStrings["noCategory"] .. craftingType) return end

    -- And now we finally do some real work.  Go through every item in the appriopriate category
    for i in pairs(cmiData[craftingType]) do
        -- If we find a match based off of the name (hopefully there's no typos in the database!) then we get it's skill information,
        --  increase the item count (janky code, but it works) and add to our tooltip string
        if cmiData[craftingType][i].name == itemInfo.name then
            itemData = cmiData[craftingType][i]
            itemCount = itemCount + 1
            
            skillminmax = ""
            if (itemData.skillmin > 0) and (itemData.skillmax > 0) then
                skillminmax = " (" .. itemData.skillmin .. " - " .. itemData.skillmax .. ")"
            end
            
			
            -- If this is our first item, don't start with a newline
            if (itemCount == 1) then
                itemString = itemString .. itemData.skill .. skillminmax --.. " ( Note: " .. itemData.note .. " )"
            else
                itemString = itemString .. "\n" .. itemData.skill .. skillminmax --.. " ( Note: " ..  itemData.note .. " )"
            end
        end
    end
    
    -- We have the right category, we just don't have the item in there for some reason.  Probably because I'm an idiot.
    if (itemCount == 0) then
        print (cmiStrings["noItem"] .. itemInfo.name .. " (" .. craftingType .. ")")
        return
    end

    -- Finally, we can make our tooltip
    makeTooltip(itemString)
end

-----------------------------------------------------------------------------------------------------
-- Tooltip generation code.  Simply positions, sizes and sets the text to display, then displays it!
-----------------------------------------------------------------------------------------------------
function makeTooltip(itemString)

-----------------------------------------------------------------------------------------------------
-- Tooltip placement new style or old style.
-----------------------------------------------------------------------------------------------------
	--[[
	 To enable remove the -- from the front of the one you would like to use.
	 only one of these can be active at a time.. 
	 
	so lets say you want to use the old style then you will find the section old style, and then remove the -- from it like so
	cmiTooltipFrame:SetPoint("BOTTOMRIGHT", UI.Native.Tooltip, "BOTTOMLEFT", 5, 0) 
	then you would goto the new style section and place -- in front of it, so now it will look like this.
	--cmiTooltipFrame:SetPoint("BOTTOMRIGHT", UI.Native.TooltipAnchor, "BOTTOMLEFT", 5, 0)
	
	
	I have this set for the new style of tooltip placement.
	
	Default Enabled
	new style
	if you have a addon that places its tool tip in the upper fight hand corner of the screen then you want this enabled.
	Attach our tooltip to the "tooltip anchor point", and not on the "tooltip" its self
    ]]--
	cmiTooltipFrame:SetPoint("BOTTOMRIGHT", UI.Native.TooltipAnchor, "BOTTOMLEFT", 5, 0)
	--[[
	Default Not Enabled.
	old style 
	if you Do not have a addon that places its tool tip in the upper fight hand corner of the screen then you want this enabled.
	Attach our tooltip to the "tooltip", and not on the "tooltip anchor point".
	]]--
	--cmiTooltipFrame:SetPoint("BOTTOMRIGHT", UI.Native.Tooltip, "BOTTOMLEFT", 5, 0)
	
	-- DO NOT EDIT BELOW HERE!!!!!!!!!
	
    cmiTooltipText:SetPoint("TOPLEFT", cmiTooltipFrame, "TOPLEFT", CMIXPADDING, CMIYPADDING)
    cmiTooltipText:SetText(itemString)
    cmiTooltipText:SetFontColor(1, 1, 1)
    cmiTooltipText:SetFontSize(13)

    cmiTooltipFrame:SetWidth(cmiTooltipText:GetWidth() + CMIXPADDING * 2)
    cmiTooltipFrame:SetHeight(cmiTooltipText:GetHeight() + CMIYPADDING * 2)

    cmiContext:SetVisible(true)
end


LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)

-- Attach to the game's Tooltip event.  There might be a better way of handling this sort of addon, but this is my first addon, so deal with it :)
Command.Event.Attach(Event.Tooltip, cmiAnalyzeItem, "Tooltip Handler")

print(string.format("v%s loaded.", addon.toc.Version))